@@ -1,5 +1,6 @@
 use pyo3::exceptions::PyTypeError;
-use pyo3::once_cell::GILOnceCell;
+use pyo3::sync::GILOnceCell;
+use std::borrow::Cow;
 use std::fmt;
 
 use pyo3::prelude::*;
@@ -11,7 +12,7 @@ use crate::lookup_key::{LookupPath, PathItem};
 
 /// Used to store individual items of the error location, e.g. a string for key/field names
 /// or a number for array indices.
-#[derive(Clone)]
+#[derive(Clone, Eq, PartialEq, IntoPyObjectRef)]
 #[cfg_attr(debug_assertions, derive(Debug))]
 pub enum LocItem {
     /// string type key, used to identify items from a dict or anything that implements `__getitem__`
@@ -19,7 +20,7 @@ pub enum LocItem {
     /// integer key, used to get:
     ///   * items from a list
     ///   * items from a tuple
-    ///   * dict with int keys `Dict[int, ...]` (python only)
+    ///   * dict with int keys `dict[int, ...]` (python only)
     ///   * with integer keys in tagged unions
     I(i64),
 }
@@ -34,24 +35,30 @@ impl fmt::Display for LocItem {
     }
 }
 
-// TODO rename to ToLocItem
-pub trait AsLocItem {
-    // TODO rename to to_loc_item
-    fn as_loc_item(&self) -> LocItem;
-}
-
 impl From<String> for LocItem {
     fn from(s: String) -> Self {
         Self::S(s)
     }
 }
 
+impl From<&String> for LocItem {
+    fn from(s: &String) -> Self {
+        s.to_string().into()
+    }
+}
+
 impl From<&str> for LocItem {
     fn from(s: &str) -> Self {
         Self::S(s.to_string())
     }
 }
 
+impl From<Cow<'_, str>> for LocItem {
+    fn from(s: Cow<'_, str>) -> Self {
+        Self::S(s.into_owned())
+    }
+}
+
 impl From<i64> for LocItem {
     fn from(i: i64) -> Self {
         Self::I(i)
@@ -78,15 +85,6 @@ impl From<PathItem> for LocItem {
     }
 }
 
-impl ToPyObject for LocItem {
-    fn to_object(&self, py: Python<'_>) -> PyObject {
-        match self {
-            Self::S(val) => val.to_object(py),
-            Self::I(val) => val.to_object(py),
-        }
-    }
-}
-
 impl Serialize for LocItem {
     fn serialize<S>(&self, serializer: S) -> Result<S::Ok, S::Error>
     where
@@ -121,15 +119,20 @@ impl Default for Location {
     }
 }
 
-static EMPTY_TUPLE: GILOnceCell<PyObject> = GILOnceCell::new();
+static EMPTY_TUPLE: GILOnceCell<Py<PyTuple>> = GILOnceCell::new();
 
-impl ToPyObject for Location {
-    fn to_object(&self, py: Python<'_>) -> PyObject {
+impl<'py> IntoPyObject<'py> for &'_ Location {
+    type Target = PyTuple;
+    type Output = Bound<'py, PyTuple>;
+    type Error = PyErr;
+
+    fn into_pyobject(self, py: Python<'py>) -> PyResult<Bound<'py, PyTuple>> {
         match self {
-            Self::List(loc) => PyTuple::new(py, loc.iter().rev()).to_object(py),
-            Self::Empty => EMPTY_TUPLE
-                .get_or_init(py, || PyTuple::empty(py).to_object(py))
-                .clone_ref(py),
+            Location::List(loc) => PyTuple::new(py, loc.iter().rev()),
+            Location::Empty => Ok(EMPTY_TUPLE
+                .get_or_init(py, || PyTuple::empty(py).unbind())
+                .bind(py)
+                .clone()),
         }
     }
 }
@@ -193,17 +196,17 @@ impl Serialize for Location {
     }
 }
 
-impl TryFrom<Option<&PyAny>> for Location {
+impl TryFrom<Option<&Bound<'_, PyAny>>> for Location {
     type Error = PyErr;
 
     /// Only ever called by ValidationError -> PyLineError to convert user input to our internal Location
     /// Thus this expects the location to *not* be reversed and reverses it before storing it.
-    fn try_from(location: Option<&PyAny>) -> PyResult<Self> {
+    fn try_from(location: Option<&Bound<'_, PyAny>>) -> PyResult<Self> {
         if let Some(location) = location {
             let mut loc_vec: Vec<LocItem> = if let Ok(tuple) = location.downcast::<PyTuple>() {
-                tuple.iter().map(AsLocItem::as_loc_item).collect()
+                tuple.iter().map(Into::into).collect()
             } else if let Ok(list) = location.downcast::<PyList>() {
-                list.iter().map(AsLocItem::as_loc_item).collect()
+                list.iter().map(Into::into).collect()
             } else {
                 return Err(PyTypeError::new_err(
                     "Location must be a list or tuple of strings and ints",
