@@ -6,15 +6,15 @@ mod types;
 mod validation_exception;
 mod value_exception;
 
-pub use self::line_error::{AsErrorValue, InputValue, ValError, ValLineError, ValResult};
-pub use self::location::{AsLocItem, LocItem};
+pub use self::line_error::{InputValue, ToErrorValue, ValError, ValLineError, ValResult};
+pub use self::location::LocItem;
 pub use self::types::{list_all_errors, ErrorType, ErrorTypeDefaults, Number};
-pub use self::validation_exception::ValidationError;
+pub use self::validation_exception::{PyLineError, ValidationError};
 pub use self::value_exception::{PydanticCustomError, PydanticKnownError, PydanticOmit, PydanticUseDefault};
 
 pub fn py_err_string(py: Python, err: PyErr) -> String {
     let value = err.value(py);
-    match value.get_type().name() {
+    match value.get_type().qualname() {
         Ok(type_name) => match value.str() {
             Ok(py_str) => {
                 let str_cow = py_str.to_string_lossy();
