@@ -5,10 +5,10 @@ use std::hash::{Hash, Hasher};
 
 use idna::punycode::decode_to_string;
 use pyo3::exceptions::PyValueError;
-use pyo3::once_cell::GILOnceCell;
 use pyo3::pyclass::CompareOp;
+use pyo3::sync::GILOnceCell;
 use pyo3::types::{PyDict, PyType};
-use pyo3::{intern, prelude::*};
+use pyo3::{intern, prelude::*, IntoPyObjectExt};
 use url::Url;
 
 use crate::tools::SchemaDict;
@@ -16,8 +16,8 @@ use crate::SchemaValidator;
 
 static SCHEMA_DEFINITION_URL: GILOnceCell<SchemaValidator> = GILOnceCell::new();
 
-#[pyclass(name = "Url", module = "pydantic_core._pydantic_core", subclass)]
-#[derive(Clone)]
+#[pyclass(name = "Url", module = "pydantic_core._pydantic_core", subclass, frozen)]
+#[derive(Clone, Hash)]
 #[cfg_attr(debug_assertions, derive(Debug))]
 pub struct PyUrl {
     lib_url: Url,
@@ -28,24 +28,24 @@ impl PyUrl {
         Self { lib_url }
     }
 
-    pub fn into_url(self) -> Url {
-        self.lib_url
+    pub fn url(&self) -> &Url {
+        &self.lib_url
     }
 }
 
 fn build_schema_validator(py: Python, schema_type: &str) -> SchemaValidator {
-    let schema: &PyDict = PyDict::new(py);
+    let schema = PyDict::new(py);
     schema.set_item("type", schema_type).unwrap();
-    SchemaValidator::py_new(py, schema, None).unwrap()
+    SchemaValidator::py_new(py, &schema, None).unwrap()
 }
 
 #[pymethods]
 impl PyUrl {
     #[new]
-    pub fn py_new(py: Python, url: &PyAny) -> PyResult<Self> {
+    pub fn py_new(py: Python, url: &Bound<'_, PyAny>) -> PyResult<Self> {
         let schema_obj = SCHEMA_DEFINITION_URL
             .get_or_init(py, || build_schema_validator(py, "url"))
-            .validate_python(py, url, None, None, None, None)?;
+            .validate_python(py, url, None, None, None, None, false.into())?;
         schema_obj.extract(py)
     }
 
@@ -98,13 +98,13 @@ impl PyUrl {
         self.lib_url.query()
     }
 
-    pub fn query_params(&self, py: Python) -> PyObject {
+    pub fn query_params<'py>(&self, py: Python<'py>) -> PyResult<Bound<'py, PyAny>> {
         // `query_pairs` is a pure iterator, so can't implement `ExactSizeIterator`, hence we need the temporary `Vec`
         self.lib_url
             .query_pairs()
-            .map(|(key, value)| (key, value).into_py(py))
-            .collect::<Vec<PyObject>>()
-            .into_py(py)
+            .map(|(key, value)| (key, value).into_pyobject(py))
+            .collect::<PyResult<Vec<_>>>()?
+            .into_pyobject(py)
     }
 
     #[getter]
@@ -138,7 +138,7 @@ impl PyUrl {
 
     fn __hash__(&self) -> u64 {
         let mut s = DefaultHasher::new();
-        self.lib_url.to_string().hash(&mut s);
+        self.hash(&mut s);
         s.finish()
     }
 
@@ -147,8 +147,8 @@ impl PyUrl {
     }
 
     #[pyo3(signature = (_memo, /))]
-    pub fn __deepcopy__(&self, py: Python, _memo: &PyDict) -> Py<PyAny> {
-        self.clone().into_py(py)
+    pub fn __deepcopy__(&self, py: Python, _memo: Bound<'_, PyAny>) -> PyResult<Py<PyAny>> {
+        self.clone().into_py_any(py)
     }
 
     fn __getnewargs__(&self) -> (&str,) {
@@ -158,8 +158,8 @@ impl PyUrl {
     #[classmethod]
     #[pyo3(signature=(*, scheme, host, username=None, password=None, port=None, path=None, query=None, fragment=None))]
     #[allow(clippy::too_many_arguments)]
-    pub fn build<'a>(
-        cls: &'a PyType,
+    pub fn build<'py>(
+        cls: &Bound<'py, PyType>,
         scheme: &str,
         host: &str,
         username: Option<&str>,
@@ -168,7 +168,7 @@ impl PyUrl {
         path: Option<&str>,
         query: Option<&str>,
         fragment: Option<&str>,
-    ) -> PyResult<&'a PyAny> {
+    ) -> PyResult<Bound<'py, PyAny>> {
         let url_host = UrlHostParts {
             username: username.map(Into::into),
             password: password.map(Into::into),
@@ -192,8 +192,8 @@ impl PyUrl {
     }
 }
 
-#[pyclass(name = "MultiHostUrl", module = "pydantic_core._pydantic_core", subclass)]
-#[derive(Clone)]
+#[pyclass(name = "MultiHostUrl", module = "pydantic_core._pydantic_core", subclass, frozen)]
+#[derive(Clone, Hash)]
 #[cfg_attr(debug_assertions, derive(Debug))]
 pub struct PyMultiHostUrl {
     ref_url: PyUrl,
@@ -208,6 +208,10 @@ impl PyMultiHostUrl {
         }
     }
 
+    pub fn lib_url(&self) -> &Url {
+        &self.ref_url.lib_url
+    }
+
     pub fn mut_lib_url(&mut self) -> &mut Url {
         &mut self.ref_url.lib_url
     }
@@ -218,10 +222,10 @@ static SCHEMA_DEFINITION_MULTI_HOST_URL: GILOnceCell<SchemaValidator> = GILOnceC
 #[pymethods]
 impl PyMultiHostUrl {
     #[new]
-    pub fn py_new(py: Python, url: &PyAny) -> PyResult<Self> {
+    pub fn py_new(py: Python, url: &Bound<'_, PyAny>) -> PyResult<Self> {
         let schema_obj = SCHEMA_DEFINITION_MULTI_HOST_URL
             .get_or_init(py, || build_schema_validator(py, "multi-host-url"))
-            .validate_python(py, url, None, None, None, None)?;
+            .validate_python(py, url, None, None, None, None, false.into())?;
         schema_obj.extract(py)
     }
 
@@ -230,7 +234,7 @@ impl PyMultiHostUrl {
         self.ref_url.scheme()
     }
 
-    pub fn hosts<'py>(&self, py: Python<'py>) -> PyResult<Vec<&'py PyDict>> {
+    pub fn hosts<'py>(&self, py: Python<'py>) -> PyResult<Vec<Bound<'py, PyDict>>> {
         if let Some(extra_urls) = &self.extra_urls {
             let mut hosts = Vec::with_capacity(extra_urls.len() + 1);
             for url in extra_urls {
@@ -255,7 +259,7 @@ impl PyMultiHostUrl {
         self.ref_url.query()
     }
 
-    pub fn query_params(&self, py: Python) -> PyObject {
+    pub fn query_params<'py>(&self, py: Python<'py>) -> PyResult<Bound<'py, PyAny>> {
         self.ref_url.query_params(py)
     }
 
@@ -338,8 +342,7 @@ impl PyMultiHostUrl {
 
     fn __hash__(&self) -> u64 {
         let mut s = DefaultHasher::new();
-        self.ref_url.clone().into_url().to_string().hash(&mut s);
-        self.extra_urls.hash(&mut s);
+        self.hash(&mut s);
         s.finish()
     }
 
@@ -347,8 +350,8 @@ impl PyMultiHostUrl {
         true // an empty string is not a valid URL
     }
 
-    pub fn __deepcopy__(&self, py: Python, _memo: &PyDict) -> Py<PyAny> {
-        self.clone().into_py(py)
+    pub fn __deepcopy__(&self, py: Python, _memo: &Bound<'_, PyDict>) -> PyResult<Py<PyAny>> {
+        self.clone().into_py_any(py)
     }
 
     fn __getnewargs__(&self) -> (String,) {
@@ -358,8 +361,8 @@ impl PyMultiHostUrl {
     #[classmethod]
     #[pyo3(signature=(*, scheme, hosts=None, path=None, query=None, fragment=None, host=None, username=None, password=None, port=None))]
     #[allow(clippy::too_many_arguments)]
-    pub fn build<'a>(
-        cls: &'a PyType,
+    pub fn build<'py>(
+        cls: &Bound<'py, PyType>,
         scheme: &str,
         hosts: Option<Vec<UrlHostParts>>,
         path: Option<&str>,
@@ -370,7 +373,7 @@ impl PyMultiHostUrl {
         username: Option<&str>,
         password: Option<&str>,
         port: Option<u16>,
-    ) -> PyResult<&'a PyAny> {
+    ) -> PyResult<Bound<'py, PyAny>> {
         let mut url =
             if hosts.is_some() && (host.is_some() || username.is_some() || password.is_some() || port.is_some()) {
                 return Err(PyValueError::new_err(
@@ -434,7 +437,7 @@ impl UrlHostParts {
 }
 
 impl FromPyObject<'_> for UrlHostParts {
-    fn extract(ob: &'_ PyAny) -> PyResult<Self> {
+    fn extract_bound(ob: &Bound<'_, PyAny>) -> PyResult<Self> {
         let py = ob.py();
         let dict = ob.downcast::<PyDict>()?;
         Ok(UrlHostParts {
@@ -464,15 +467,9 @@ impl fmt::Display for UrlHostParts {
     }
 }
 
-fn host_to_dict<'a>(py: Python<'a>, lib_url: &Url) -> PyResult<&'a PyDict> {
+fn host_to_dict<'a>(py: Python<'a>, lib_url: &Url) -> PyResult<Bound<'a, PyDict>> {
     let dict = PyDict::new(py);
-    dict.set_item(
-        "username",
-        match lib_url.username() {
-            "" => py.None(),
-            user => user.into_py(py),
-        },
-    )?;
+    dict.set_item("username", Some(lib_url.username()).filter(|s| !s.is_empty()))?;
     dict.set_item("password", lib_url.password())?;
     dict.set_item("host", lib_url.host_str())?;
     dict.set_item("port", lib_url.port_or_known_default())?;
