@@ -1,6 +1,7 @@
 """
 Numerous benchmarks of specific functionality.
 """
+
 import decimal
 import json
 import platform
@@ -8,7 +9,7 @@ import sys
 from datetime import date, datetime, timedelta, timezone
 from decimal import Decimal
 from enum import Enum
-from typing import Any, List
+from typing import Any
 from uuid import UUID
 
 import pytest
@@ -324,7 +325,7 @@ def test_definition_model_core(definition_model_data, benchmark):
     benchmark(v.validate_python, definition_model_data)
 
 
-@pytest.mark.benchmark(group='List[TypedDict]')
+@pytest.mark.benchmark(group='list[TypedDict]')
 def test_list_of_dict_models_core(benchmark):
     v = SchemaValidator(
         {
@@ -343,7 +344,7 @@ def test_list_of_dict_models_core(benchmark):
 list_of_ints_data = ([i for i in range(1000)], [str(i) for i in range(1000)])
 
 
-@pytest.mark.benchmark(group='List[int]')
+@pytest.mark.benchmark(group='list[int]')
 def test_list_of_ints_core_py(benchmark):
     v = SchemaValidator({'type': 'list', 'items_schema': {'type': 'int'}})
 
@@ -353,7 +354,7 @@ def test_list_of_ints_core_py(benchmark):
         v.validate_python(list_of_ints_data[1])
 
 
-@pytest.mark.benchmark(group='List[int] JSON')
+@pytest.mark.benchmark(group='list[int] JSON')
 def test_list_of_ints_core_json(benchmark):
     v = SchemaValidator({'type': 'list', 'items_schema': {'type': 'int'}})
 
@@ -365,7 +366,33 @@ def test_list_of_ints_core_json(benchmark):
         v.validate_json(json_data[1])
 
 
-@pytest.mark.benchmark(group='List[Any]')
+list_of_strs_data = [str(i) for i in range(1000)]
+
+
+@pytest.mark.benchmark(group='list[str]')
+def test_list_of_strs_py_cached(benchmark):
+    v = SchemaValidator(core_schema.list_schema(core_schema.str_schema()))
+
+    benchmark(v.validate_python, list_of_strs_data)
+
+
+@pytest.mark.benchmark(group='list[str]')
+def test_list_of_strs_json_cached(benchmark):
+    v = SchemaValidator(core_schema.list_schema(core_schema.str_schema()))
+
+    json_data = json.dumps(list_of_strs_data)
+    benchmark(v.validate_json, json_data)
+
+
+@pytest.mark.benchmark(group='list[str]')
+def test_list_of_strs_json_uncached(benchmark):
+    v = SchemaValidator(core_schema.list_schema(core_schema.str_schema()), {'cache_strings': False})
+
+    json_data = json.dumps(list_of_strs_data)
+    benchmark(v.validate_json, json_data)
+
+
+@pytest.mark.benchmark(group='list[Any]')
 def test_list_of_any_core_py(benchmark):
     v = SchemaValidator({'type': 'list'})
 
@@ -454,7 +481,7 @@ def test_frozenset_of_ints_duplicates_core(benchmark):
 dict_of_ints_data = ({str(i): i for i in range(1000)}, {str(i): str(i) for i in range(1000)})
 
 
-@pytest.mark.benchmark(group='Dict[str, int]')
+@pytest.mark.benchmark(group='dict[str, int]')
 def test_dict_of_ints_core(benchmark):
     v = SchemaValidator({'type': 'dict', 'keys_schema': {'type': 'str'}, 'values_schema': {'type': 'int'}})
 
@@ -464,7 +491,7 @@ def test_dict_of_ints_core(benchmark):
         v.validate_python(dict_of_ints_data[1])
 
 
-@pytest.mark.benchmark(group='Dict[any, any]')
+@pytest.mark.benchmark(group='dict[any, any]')
 def test_dict_of_any_core(benchmark):
     v = SchemaValidator({'type': 'dict'})
 
@@ -474,7 +501,7 @@ def test_dict_of_any_core(benchmark):
         v.validate_python(dict_of_ints_data[1])
 
 
-@pytest.mark.benchmark(group='Dict[str, int] JSON')
+@pytest.mark.benchmark(group='dict[str, int] JSON')
 def test_dict_of_ints_core_json(benchmark):
     v = SchemaValidator({'type': 'dict', 'keys_schema': {'type': 'str'}, 'values_schema': {'type': 'int'}})
 
@@ -489,7 +516,7 @@ def test_dict_of_ints_core_json(benchmark):
 many_models_data = [{'age': i} for i in range(1000)]
 
 
-@pytest.mark.benchmark(group='List[DictSimpleMode]')
+@pytest.mark.benchmark(group='list[DictSimpleMode]')
 def test_many_models_core_dict(benchmark):
     model_schema = {
         'type': 'list',
@@ -502,7 +529,7 @@ def test_many_models_core_dict(benchmark):
     benchmark(v.validate_python, many_models_data)
 
 
-@pytest.mark.benchmark(group='List[SimpleMode]')
+@pytest.mark.benchmark(group='list[SimpleMode]')
 def test_many_models_core_model(benchmark):
     class MyCoreModel:
         __slots__ = '__dict__', '__pydantic_fields_set__', '__pydantic_extra__', '__pydantic_private__'
@@ -526,7 +553,7 @@ def test_many_models_core_model(benchmark):
 list_of_nullable_data = [None if i % 2 else i for i in range(1000)]
 
 
-@pytest.mark.benchmark(group='List[Nullable[int]]')
+@pytest.mark.benchmark(group='list[Nullable[int]]')
 def test_list_of_nullable_core(benchmark):
     v = SchemaValidator({'type': 'list', 'items_schema': {'type': 'nullable', 'schema': {'type': 'int'}}})
 
@@ -634,13 +661,13 @@ class TestBenchmarkDateX:
     def test_core_future(self, benchmark):
         v = SchemaValidator({'type': 'date', 'gt': date.today()})
 
-        benchmark(v.validate_python, date(2032, 1, 1))
+        benchmark(v.validate_python, date(2932, 1, 1))
 
     @pytest.mark.benchmark(group='date future')
     def test_core_future_str(self, benchmark):
         v = SchemaValidator({'type': 'date', 'gt': date.today()})
 
-        benchmark(v.validate_python, str(date(2032, 1, 1)))
+        benchmark(v.validate_python, str(date(2932, 1, 1)))
 
 
 class TestBenchmarkUnion:
@@ -848,7 +875,7 @@ def test_raise_error_custom(benchmark):
 def test_positional_tuple(benchmark):
     v = SchemaValidator(
         {
-            'type': 'tuple-positional',
+            'type': 'tuple',
             'items_schema': [{'type': 'int'}, {'type': 'int'}, {'type': 'int'}, {'type': 'int'}, {'type': 'int'}],
         }
     )
@@ -859,7 +886,7 @@ def test_positional_tuple(benchmark):
 
 @pytest.mark.benchmark(group='tuple')
 def test_variable_tuple(benchmark):
-    v = SchemaValidator({'type': 'tuple-variable', 'items_schema': {'type': 'int'}})
+    v = SchemaValidator({'type': 'tuple', 'items_schema': [{'type': 'int'}], 'variadic_item_index': 0})
     assert v.validate_python((1, 2, 3, '4', 5)) == (1, 2, 3, 4, 5)
 
     benchmark(v.validate_python, (1, 2, 3, '4', 5))
@@ -867,7 +894,7 @@ def test_variable_tuple(benchmark):
 
 @pytest.mark.benchmark(group='tuple-many')
 def test_tuple_many_variable(benchmark):
-    v = SchemaValidator({'type': 'tuple-variable', 'items_schema': {'type': 'int'}})
+    v = SchemaValidator({'type': 'tuple', 'items_schema': [{'type': 'int'}], 'variadic_item_index': 0})
     assert v.validate_python(list(range(10))) == tuple(range(10))
 
     benchmark(v.validate_python, list(range(10)))
@@ -875,7 +902,7 @@ def test_tuple_many_variable(benchmark):
 
 @pytest.mark.benchmark(group='tuple-many')
 def test_tuple_many_positional(benchmark):
-    v = SchemaValidator({'type': 'tuple-positional', 'items_schema': [], 'extras_schema': {'type': 'int'}})
+    v = SchemaValidator({'type': 'tuple', 'items_schema': [{'type': 'int'}], 'variadic_item_index': 0})
     assert v.validate_python(list(range(10))) == tuple(range(10))
 
     benchmark(v.validate_python, list(range(10)))
@@ -1194,7 +1221,7 @@ LARGE_STR_PREFIX = 'a' * 50
 )
 @pytest.mark.parametrize('py_or_json', ['python', 'json'])
 def test_validate_literal(
-    benchmark: Any, allowed_values: List[Any], input: Any, expected_val_res: Any, py_or_json: str
+    benchmark: Any, allowed_values: list[Any], input: Any, expected_val_res: Any, py_or_json: str
 ) -> None:
     validator = SchemaValidator(core_schema.literal_schema(expected=allowed_values))
 
@@ -1215,7 +1242,7 @@ def test_validate_literal(
 def test_core_root_model(benchmark):
     class MyModel:
         __slots__ = '__dict__', '__pydantic_fields_set__', '__pydantic_extra__', '__pydantic_private__'
-        root: List[int]
+        root: list[int]
 
     v = SchemaValidator(
         core_schema.model_schema(MyModel, core_schema.list_schema(core_schema.int_schema()), root_model=True)
@@ -1232,6 +1259,18 @@ def test_strict_int(benchmark):
     benchmark(v.validate_python, 42)
 
 
+@pytest.mark.benchmark(group='strict_int')
+def test_strict_int_fails(benchmark):
+    v = SchemaValidator(core_schema.int_schema(strict=True))
+
+    @benchmark
+    def t():
+        try:
+            v.validate_python(())
+        except ValidationError:
+            pass
+
+
 @pytest.mark.benchmark(group='int_range')
 def test_int_range(benchmark):
     v = SchemaValidator(core_schema.int_schema(gt=0, lt=100))
@@ -1373,3 +1412,63 @@ class TestBenchmarkDecimal:
     @pytest.mark.benchmark(group='decimal from str')
     def test_decimal_from_string_limit(self, benchmark):
         benchmark(decimal.Decimal, '123.456789')
+
+
+class FooInt(int, Enum):
+    a = 1
+    b = 2
+    c = 3
+
+
+@pytest.mark.benchmark(group='enum int')
+def test_enum_int_python(benchmark):
+    def to_enum(input_value: Any, /) -> Enum:
+        try:
+            return FooInt(input_value)
+        except ValueError:
+            raise PydanticCustomError('enum', 'Input should be {expected}', {'expected': '1, 2 or 3'})
+
+    v = SchemaValidator(core_schema.no_info_after_validator_function(to_enum, core_schema.int_schema()))
+
+    assert v.validate_python(1) is FooInt.a
+
+    benchmark(v.validate_python, 1)
+
+
+@pytest.mark.benchmark(group='enum int')
+def test_enum_int_core(benchmark):
+    v = SchemaValidator(core_schema.enum_schema(FooInt, list(FooInt.__members__.values()), sub_type='int'))
+
+    assert v.validate_python(1) is FooInt.a
+
+    benchmark(v.validate_python, 1)
+
+
+class FooStr(str, Enum):
+    a = 'apple'
+    b = 'banana'
+    c = 'carrot'
+
+
+@pytest.mark.benchmark(group='enum str')
+def test_enum_str_python(benchmark):
+    def to_enum(input_value: Any, /) -> Enum:
+        try:
+            return FooStr(input_value)
+        except ValueError:
+            raise PydanticCustomError('enum', 'Input should be {expected}', {'expected': 'apple, banana or carrot'})
+
+    v = SchemaValidator(core_schema.no_info_after_validator_function(to_enum, core_schema.str_schema()))
+
+    assert v.validate_python('apple') is FooStr.a
+
+    benchmark(v.validate_python, 'apple')
+
+
+@pytest.mark.benchmark(group='enum str')
+def test_enum_str_core(benchmark):
+    v = SchemaValidator(core_schema.enum_schema(FooStr, list(FooStr.__members__.values()), sub_type='str'))
+
+    assert v.validate_python('apple') is FooStr.a
+
+    benchmark(v.validate_python, 'apple')
