@@ -1,6 +1,7 @@
 import gc
 import platform
-from typing import Any, Iterable
+from collections.abc import Iterable
+from typing import Any
 from weakref import WeakValueDictionary
 
 import pytest
@@ -28,10 +29,10 @@ def test_gc_schema_serializer() -> None:
 
         def __init_subclass__(cls) -> None:
             cls.__schema__ = SchemaSerializer(
-                core_schema.model_schema(cls, GC_TEST_SCHEMA_INNER), config={'ser_json_timedelta': 'seconds_float'}
+                core_schema.model_schema(cls, GC_TEST_SCHEMA_INNER), config={'ser_json_timedelta': 'float'}
             )
 
-    cache: 'WeakValueDictionary[int, Any]' = WeakValueDictionary()
+    cache: WeakValueDictionary[int, Any] = WeakValueDictionary()
 
     for _ in range(10_000):
 
@@ -63,7 +64,7 @@ def test_gc_schema_validator() -> None:
                 config=core_schema.CoreConfig(extra_fields_behavior='allow'),
             )
 
-    cache: 'WeakValueDictionary[int, Any]' = WeakValueDictionary()
+    cache: WeakValueDictionary[int, Any] = WeakValueDictionary()
 
     for _ in range(10_000):
 
@@ -105,7 +106,7 @@ def test_gc_validator_iterator() -> None:
         def __next__(self):
             raise StopIteration()
 
-    cache: 'WeakValueDictionary[int, Any]' = WeakValueDictionary()
+    cache: WeakValueDictionary[int, Any] = WeakValueDictionary()
 
     for _ in range(10_000):
         iterable = MyIterable()
