@@ -1,7 +1,6 @@
 import json
 import platform
 import re
-from typing import List
 
 import pytest
 from dirty_equals import IsFloatNan, IsList
@@ -13,6 +12,7 @@ from pydantic_core import (
     SchemaValidator,
     ValidationError,
     core_schema,
+    from_json,
     to_json,
     to_jsonable_python,
 )
@@ -248,7 +248,7 @@ def test_to_jsonable_python_fallback():
 
 def test_to_jsonable_python_schema_serializer():
     class Foobar:
-        def __init__(self, my_foo: int, my_inners: List['Foobar']):
+        def __init__(self, my_foo: int, my_inners: list['Foobar']):
             self.my_foo = my_foo
             self.my_inners = my_inners
 
@@ -365,3 +365,101 @@ def test_inf_nan_allow():
     assert v.validate_json('Infinity') == float('inf')
     assert v.validate_json('-Infinity') == float('-inf')
     assert v.validate_json('NaN') == IsFloatNan()
+
+
+def test_partial_parse():
+    with pytest.raises(ValueError, match='EOF while parsing a string at line 1 column 15'):
+        from_json('["aa", "bb", "c')
+    assert from_json('["aa", "bb", "c', allow_partial=True) == ['aa', 'bb']
+
+    with pytest.raises(ValueError, match='EOF while parsing a string at line 1 column 15'):
+        from_json(b'["aa", "bb", "c')
+    assert from_json(b'["aa", "bb", "c', allow_partial=True) == ['aa', 'bb']
+
+
+def test_json_bytes_base64_round_trip():
+    data = b'\xd8\x07\xc1Tx$\x91F%\xf3\xf3I\xca\xd8@\x0c\xee\xc3\xab\xff\x7f\xd3\xcd\xcd\xf9\xc2\x10\xe4\xa1\xb01e'
+    encoded_std = b'"2AfBVHgkkUYl8/NJythADO7Dq/9/083N+cIQ5KGwMWU="'
+    encoded_url = b'"2AfBVHgkkUYl8_NJythADO7Dq_9_083N-cIQ5KGwMWU="'
+    assert to_json(data, bytes_mode='base64') == encoded_url
+
+    v = SchemaValidator({'type': 'bytes'}, {'val_json_bytes': 'base64'})
+    assert v.validate_json(encoded_url) == data
+    assert v.validate_json(encoded_std) == data
+
+    with pytest.raises(ValidationError) as exc:
+        v.validate_json('"wrong!"')
+    [details] = exc.value.errors()
+    assert details['type'] == 'bytes_invalid_encoding'
+
+    assert to_json({'key': data}, bytes_mode='base64') == b'{"key":' + encoded_url + b'}'
+    v = SchemaValidator(
+        {'type': 'dict', 'keys_schema': {'type': 'str'}, 'values_schema': {'type': 'bytes'}},
+        {'val_json_bytes': 'base64'},
+    )
+    assert v.validate_json(b'{"key":' + encoded_url + b'}') == {'key': data}
+
+
+def test_json_bytes_base64_no_padding():
+    v = SchemaValidator({'type': 'bytes'}, {'val_json_bytes': 'base64'})
+    base_64_without_padding = 'bm8tcGFkZGluZw'
+    assert v.validate_json(json.dumps(base_64_without_padding)) == b'no-padding'
+
+
+def test_json_bytes_base64_invalid():
+    v = SchemaValidator({'type': 'bytes'}, {'val_json_bytes': 'base64'})
+    wrong_input = 'wrong!'
+    with pytest.raises(ValidationError) as exc_info:
+        v.validate_json(json.dumps(wrong_input))
+    assert exc_info.value.errors(include_url=False, include_context=False) == [
+        {
+            'type': 'bytes_invalid_encoding',
+            'loc': (),
+            'msg': f'Data should be valid base64: Invalid symbol {ord("!")}, offset {len(wrong_input) - 1}.',
+            'input': wrong_input,
+        }
+    ]
+
+
+def test_json_bytes_hex_round_trip():
+    data = b'hello'
+    encoded = b'"68656c6c6f"'
+    assert to_json(data, bytes_mode='hex') == encoded
+
+    v = SchemaValidator({'type': 'bytes'}, {'val_json_bytes': 'hex'})
+    assert v.validate_json(encoded) == data
+
+    assert to_json({'key': data}, bytes_mode='hex') == b'{"key":"68656c6c6f"}'
+    v = SchemaValidator(
+        {'type': 'dict', 'keys_schema': {'type': 'str'}, 'values_schema': {'type': 'bytes'}},
+        {'val_json_bytes': 'hex'},
+    )
+    assert v.validate_json('{"key":"68656c6c6f"}') == {'key': data}
+
+
+def test_json_bytes_hex_invalid():
+    v = SchemaValidator({'type': 'bytes'}, {'val_json_bytes': 'hex'})
+
+    wrong_input = 'a'
+    with pytest.raises(ValidationError) as exc_info:
+        v.validate_json(json.dumps(wrong_input))
+    assert exc_info.value.errors(include_url=False, include_context=False) == [
+        {
+            'type': 'bytes_invalid_encoding',
+            'loc': (),
+            'msg': 'Data should be valid hex: Odd number of digits',
+            'input': wrong_input,
+        }
+    ]
+
+    wrong_input = 'ag'
+    with pytest.raises(ValidationError) as exc_info:
+        v.validate_json(json.dumps(wrong_input))
+    assert exc_info.value.errors(include_url=False, include_context=False) == [
+        {
+            'type': 'bytes_invalid_encoding',
+            'loc': (),
+            'msg': "Data should be valid hex: Invalid character 'g' at position 1",
+            'input': wrong_input,
+        }
+    ]
