@@ -4,7 +4,7 @@ import platform
 import re
 import sys
 import weakref
-from typing import Any, ClassVar, Dict, List, Optional, Union
+from typing import Any, ClassVar, Optional, Union
 
 import pytest
 from dirty_equals import IsListOrTuple, IsStr
@@ -968,7 +968,7 @@ def test_frozen_field():
         (core_schema.CoreConfig(extra_fields_behavior='allow'), {'extra_behavior': 'ignore'}),
     ],
 )
-def test_extra_behavior_ignore(config: Union[core_schema.CoreConfig, None], schema_extra_behavior_kw: Dict[str, Any]):
+def test_extra_behavior_ignore(config: Union[core_schema.CoreConfig, None], schema_extra_behavior_kw: dict[str, Any]):
     @dataclasses.dataclass
     class MyModel:
         f: str
@@ -1016,7 +1016,7 @@ def test_extra_behavior_ignore(config: Union[core_schema.CoreConfig, None], sche
         (core_schema.CoreConfig(extra_fields_behavior='ignore'), {'extra_behavior': 'forbid'}),
     ],
 )
-def test_extra_behavior_forbid(config: Union[core_schema.CoreConfig, None], schema_extra_behavior_kw: Dict[str, Any]):
+def test_extra_behavior_forbid(config: Union[core_schema.CoreConfig, None], schema_extra_behavior_kw: dict[str, Any]):
     @dataclasses.dataclass
     class MyModel:
         f: str
@@ -1062,7 +1062,7 @@ def test_extra_behavior_forbid(config: Union[core_schema.CoreConfig, None], sche
         (core_schema.CoreConfig(extra_fields_behavior='forbid'), {'extra_behavior': 'allow'}),
     ],
 )
-def test_extra_behavior_allow(config: Union[core_schema.CoreConfig, None], schema_extra_behavior_kw: Dict[str, Any]):
+def test_extra_behavior_allow(config: Union[core_schema.CoreConfig, None], schema_extra_behavior_kw: dict[str, Any]):
     @dataclasses.dataclass
     class MyModel:
         f: str
@@ -1090,7 +1090,7 @@ def test_extra_behavior_allow(config: Union[core_schema.CoreConfig, None], schem
 
 
 def test_function_validator_wrapping_args_schema_after() -> None:
-    calls: List[Any] = []
+    calls: list[Any] = []
 
     def func(*args: Any) -> Any:
         calls.append(args)
@@ -1122,7 +1122,7 @@ def test_function_validator_wrapping_args_schema_after() -> None:
 
 
 def test_function_validator_wrapping_args_schema_before() -> None:
-    calls: List[Any] = []
+    calls: list[Any] = []
 
     def func(*args: Any) -> Any:
         calls.append(args)
@@ -1154,7 +1154,7 @@ def test_function_validator_wrapping_args_schema_before() -> None:
 
 
 def test_function_validator_wrapping_args_schema_wrap() -> None:
-    calls: List[Any] = []
+    calls: list[Any] = []
 
     def func(*args: Any) -> Any:
         assert len(args) == 2
@@ -1510,6 +1510,27 @@ def test_dataclass_json():
     ]
 
 
+def test_dataclass_wrap_json():
+    # https://github.com/pydantic/pydantic/issues/8147
+    schema = core_schema.no_info_wrap_validator_function(
+        lambda v, handler: handler(v),
+        core_schema.dataclass_schema(
+            FooDataclass,
+            core_schema.dataclass_args_schema(
+                'FooDataclass',
+                [
+                    core_schema.dataclass_field(name='a', schema=core_schema.str_schema()),
+                    core_schema.dataclass_field(name='b', schema=core_schema.bool_schema()),
+                ],
+            ),
+            ['a', 'b'],
+        ),
+    )
+    v = SchemaValidator(schema)
+    assert v.validate_json('{"a": "hello", "b": true}') == FooDataclass(a='hello', b=True)
+    assert v.validate_json('{"a": "hello", "b": true}', strict=True) == FooDataclass(a='hello', b=True)
+
+
 @pytest.mark.xfail(
     condition=platform.python_implementation() == 'PyPy', reason='https://foss.heptapod.net/pypy/pypy/-/issues/3899'
 )
@@ -1571,3 +1592,124 @@ def test_leak_dataclass(validator):
     gc.collect()
 
     assert ref() is None
+
+
+init_test_cases = [
+    ({'a': 'hello', 'b': 'bye'}, 'ignore', {'a': 'hello', 'b': 'HELLO'}),
+    ({'a': 'hello'}, 'ignore', {'a': 'hello', 'b': 'HELLO'}),
+    # note, for the case below, we don't actually support this case in Pydantic
+    # it's disallowed in Pydantic to have a model with extra='allow' and a field
+    # with init=False, so this case isn't really possible at the momment
+    # however, no conflict arises here because we don't pass in the value for b
+    # to __init__
+    ({'a': 'hello'}, 'allow', {'a': 'hello', 'b': 'HELLO'}),
+    (
+        {'a': 'hello', 'b': 'bye'},
+        'forbid',
+        Err(
+            'Unexpected keyword argument',
+            errors=[
+                {
+                    'type': 'unexpected_keyword_argument',
+                    'loc': ('b',),
+                    'msg': 'Unexpected keyword argument',
+                    'input': 'bye',
+                }
+            ],
+        ),
+    ),
+    ({'a': 'hello'}, 'forbid', {'a': 'hello', 'b': 'HELLO'}),
+]
+
+
+@pytest.mark.parametrize(
+    'input_value,extra_behavior,expected',
+    [
+        *init_test_cases,
+        # special case - when init=False, extra='allow', and the value is provided
+        # currently, it's disallowed in Pydantic to have a model with extra='allow'
+        # and a field with init=False, so this case isn't really possible at the momment
+        # TODO: open to changing this behavior, and changes won't be significantly breaking
+        # because we currently don't support this case
+        ({'a': 'hello', 'b': 'bye'}, 'allow', {'a': 'hello', 'b': 'HELLO'}),
+    ],
+)
+def test_dataclass_args_init(input_value, extra_behavior, expected):
+    @dataclasses.dataclass
+    class Foo:
+        a: str
+        b: str
+
+        def __post_init__(self):
+            self.b = self.a.upper()
+
+    schema = core_schema.dataclass_schema(
+        Foo,
+        core_schema.dataclass_args_schema(
+            'Foo',
+            [
+                core_schema.dataclass_field(name='a', schema=core_schema.str_schema()),
+                core_schema.dataclass_field(name='b', schema=core_schema.str_schema(), init=False),
+            ],
+            extra_behavior=extra_behavior,
+        ),
+        ['a', 'b'],
+        post_init=True,
+    )
+
+    v = SchemaValidator(schema)
+
+    if isinstance(expected, Err):
+        with pytest.raises(ValidationError, match=re.escape(expected.message)) as exc_info:
+            v.validate_python(input_value)
+
+        if expected.errors is not None:
+            assert exc_info.value.errors(include_url=False) == expected.errors
+    else:
+        assert dataclasses.asdict(v.validate_python(input_value)) == expected
+
+
+@pytest.mark.parametrize(
+    'input_value,extra_behavior,expected',
+    [
+        *init_test_cases,
+        # special case - allow override of default, even when init=False, if extra='allow'
+        # TODO: we haven't really decided if this should be allowed or not
+        # currently, it's disallowed in Pydantic to have a model with extra='allow'
+        # and a field with init=False, so this case isn't really possible at the momment
+        ({'a': 'hello', 'b': 'bye'}, 'allow', {'a': 'hello', 'b': 'bye'}),
+    ],
+)
+def test_dataclass_args_init_with_default(input_value, extra_behavior, expected):
+    @dataclasses.dataclass
+    class Foo:
+        a: str
+        b: str
+
+    schema = core_schema.dataclass_schema(
+        Foo,
+        core_schema.dataclass_args_schema(
+            'Foo',
+            [
+                core_schema.dataclass_field(name='a', schema=core_schema.str_schema()),
+                core_schema.dataclass_field(
+                    name='b',
+                    schema=core_schema.with_default_schema(schema=core_schema.str_schema(), default='HELLO'),
+                    init=False,
+                ),
+            ],
+            extra_behavior=extra_behavior,
+        ),
+        ['a', 'b'],
+    )
+
+    v = SchemaValidator(schema)
+
+    if isinstance(expected, Err):
+        with pytest.raises(ValidationError, match=re.escape(expected.message)) as exc_info:
+            v.validate_python(input_value)
+
+        if expected.errors is not None:
+            assert exc_info.value.errors(include_url=False) == expected.errors
+    else:
+        assert dataclasses.asdict(v.validate_python(input_value)) == expected
