@@ -1,8 +1,9 @@
 import collections.abc
 import re
 from collections import deque
+from collections.abc import Iterator
 from dataclasses import dataclass
-from typing import Any, Dict, Iterator, List, Union
+from typing import Any, Union
 
 import pytest
 from dirty_equals import Contains, HasRepr, IsInstance, IsList, IsStr
@@ -43,7 +44,7 @@ def test_list_strict():
 
 def test_list_no_copy():
     v = SchemaValidator({'type': 'list'})
-    assert v.validate_python([1, 2, 3]) is not [1, 2, 3]
+    assert v.validate_python([1, 2, 3]) is not [1, 2, 3]  # noqa: F632
 
 
 def gen_ints():
@@ -169,7 +170,7 @@ def test_list_error(input_value, index):
         ),
     ],
 )
-def test_list_length_constraints(kwargs: Dict[str, Any], input_value, expected):
+def test_list_length_constraints(kwargs: dict[str, Any], input_value, expected):
     v = SchemaValidator({'type': 'list', **kwargs})
     if isinstance(expected, Err):
         with pytest.raises(ValidationError, match=re.escape(expected.message)):
@@ -397,8 +398,53 @@ def test_max_length_fail_fast(error_in_func: bool) -> None:
     )
 
 
+@pytest.mark.parametrize(
+    'fail_fast,expected',
+    [
+        pytest.param(
+            True,
+            [
+                {
+                    'type': 'int_parsing',
+                    'loc': (1,),
+                    'msg': 'Input should be a valid integer, unable to parse string as an integer',
+                    'input': 'not-num',
+                }
+            ],
+            id='fail_fast',
+        ),
+        pytest.param(
+            False,
+            [
+                {
+                    'type': 'int_parsing',
+                    'loc': (1,),
+                    'msg': 'Input should be a valid integer, unable to parse string as an integer',
+                    'input': 'not-num',
+                },
+                {
+                    'type': 'int_parsing',
+                    'loc': (2,),
+                    'msg': 'Input should be a valid integer, unable to parse string as an integer',
+                    'input': 'again',
+                },
+            ],
+            id='not_fail_fast',
+        ),
+    ],
+)
+def test_list_fail_fast(fail_fast, expected):
+    s = core_schema.list_schema(core_schema.int_schema(), fail_fast=fail_fast)
+    v = SchemaValidator(s)
+
+    with pytest.raises(ValidationError) as exc_info:
+        v.validate_python([1, 'not-num', 'again'])
+
+    assert exc_info.value.errors(include_url=False) == expected
+
+
 class MySequence(collections.abc.Sequence):
-    def __init__(self, data: List[Any]):
+    def __init__(self, data: list[Any]):
         self._data = data
 
     def __getitem__(self, index: int) -> Any:
@@ -412,7 +458,7 @@ class MySequence(collections.abc.Sequence):
 
 
 class MyMapping(collections.abc.Mapping):
-    def __init__(self, data: Dict[Any, Any]) -> None:
+    def __init__(self, data: dict[Any, Any]) -> None:
         self._data = data
 
     def __getitem__(self, key: Any) -> Any:
@@ -435,7 +481,7 @@ class ListInputTestCase:
     strict: Union[bool, None] = None
 
 
-LAX_MODE_INPUTS: List[Any] = [
+LAX_MODE_INPUTS: list[Any] = [
     (1, 2, 3),
     frozenset((1, 2, 3)),
     {1, 2, 3},
