@@ -1,6 +1,6 @@
 import re
 from enum import Enum
-from typing import Any, Callable, List
+from typing import Any, Callable
 
 import pytest
 
@@ -78,7 +78,7 @@ from ..conftest import Err, PyAndJson, plain_repr
         pytest.param(
             ['a', 'b'],
             'c',
-            Err("Input should be 'a' or 'b' [type=literal_error, input_value=\'c\', input_type=str]"),
+            Err("Input should be 'a' or 'b' [type=literal_error, input_value='c', input_type=str]"),
             id='wrong-multiple-str',
         ),
         ([1, '1'], 1, 1),
@@ -317,7 +317,7 @@ def test_int_enum_values():
         ),
     ],
 )
-def test_mix_int_enum_with_int(reverse: Callable[[List[Any]], List[Any]], err: Any):
+def test_mix_int_enum_with_int(reverse: Callable[[list[Any]], list[Any]], err: Any):
     class Foo(int, Enum):
         foo = 1
 
@@ -363,7 +363,7 @@ def test_mix_int_enum_with_int(reverse: Callable[[List[Any]], List[Any]], err: A
         ),
     ],
 )
-def test_mix_str_enum_with_str(reverse: Callable[[List[Any]], List[Any]], err: Any):
+def test_mix_str_enum_with_str(reverse: Callable[[list[Any]], list[Any]], err: Any):
     class Foo(str, Enum):
         foo = 'foo_val'
 
@@ -378,3 +378,26 @@ def test_mix_str_enum_with_str(reverse: Callable[[List[Any]], List[Any]], err: A
     with pytest.raises(ValidationError) as exc_info:
         v.validate_python('bar_val')
     assert exc_info.value.errors(include_url=False) == err
+
+
+def test_big_int():
+    big_int = 2**64 + 1
+    massive_int = 2**128 + 1
+    v = SchemaValidator(core_schema.literal_schema([big_int, massive_int]))
+    assert v.validate_python(big_int) == big_int
+    assert v.validate_python(massive_int) == massive_int
+    m = r'Input should be 18446744073709551617 or 340282366920938463463374607431768211457 \[type=literal_error'
+    with pytest.raises(ValidationError, match=m):
+        v.validate_python(37)
+
+
+def test_enum_for_str() -> None:
+    class S(str, Enum):
+        a = 'a'
+
+    val_enum = SchemaValidator(core_schema.literal_schema([S.a]))
+    val_str = SchemaValidator(core_schema.literal_schema(['a']))
+
+    for val in [val_enum, val_str]:
+        assert val.validate_python('a') == 'a'
+        assert val.validate_python(S.a) == 'a'
