@@ -1,6 +1,6 @@
 import re
 from copy import deepcopy
-from typing import Dict, Optional, Union
+from typing import Optional, Union
 
 import pytest
 from dirty_equals import HasRepr, IsInstance
@@ -287,6 +287,12 @@ def test_url_defaults_single_url(validator_kwargs, url, expected):
     url_test_case_helper(url, expected, SCHEMA_VALIDATOR_MODE, s)
 
 
+def test_url_host_required():
+    s = SchemaValidator(core_schema.url_schema(host_required=True))
+    url_test_case_helper('test:', Err('empty host'), SCHEMA_VALIDATOR_MODE, s)
+    url_test_case_helper('sftp://', Err('empty host'), SCHEMA_VALIDATOR_MODE, s)
+
+
 @pytest.mark.parametrize(
     'validator_kwargs,url,expected',
     [
@@ -495,7 +501,7 @@ def test_url_to_url(url_validator, multi_host_url_validator):
     url2 = url_validator.validate_python(url)
     assert isinstance(url2, Url)
     assert str(url2) == 'https://example.com/'
-    assert url is not url2
+    assert url is url2
 
     multi_url = multi_host_url_validator.validate_python('https://example.com')
     assert isinstance(multi_url, MultiHostUrl)
@@ -982,7 +988,7 @@ def test_url_to_multi_url(url_validator, multi_host_url_validator):
     url3 = multi_host_url_validator.validate_python(url2)
     assert isinstance(url3, MultiHostUrl)
     assert str(url3) == 'https://example.com/'
-    assert url2 is not url3
+    assert url2 is url3
 
 
 def test_multi_wrong_type(multi_host_url_validator):
@@ -1163,7 +1169,7 @@ def test_multi_host_url_bool() -> None:
 
 
 def test_multi_host_url_hash() -> None:
-    data: Dict[MultiHostUrl, int] = {}
+    data: dict[MultiHostUrl, int] = {}
 
     data[MultiHostUrl('http://example.com,www.example.com')] = 1
     assert data == {MultiHostUrl('http://example.com,www.example.com/'): 1}
@@ -1202,7 +1208,7 @@ def test_url_bool() -> None:
 
 
 def test_url_hash() -> None:
-    data: Dict[Url, int] = {}
+    data: dict[Url, int] = {}
 
     data[Url('http://example.com')] = 1
     assert data == {Url('http://example.com/'): 1}
@@ -1216,3 +1222,86 @@ def test_url_hash() -> None:
 
 def test_url_deepcopy() -> None:
     assert deepcopy(Url('http://example.com')) == Url('http://example.com/')
+
+
+def test_multi_url_build() -> None:
+    url = MultiHostUrl.build(
+        scheme='postgresql',
+        username='testuser',
+        password='testpassword',
+        host='127.0.0.1',
+        port=5432,
+        path='database',
+        query='sslmode=require',
+        fragment='test',
+    )
+    assert url == MultiHostUrl('postgresql://testuser:testpassword@127.0.0.1:5432/database?sslmode=require#test')
+    assert str(url) == 'postgresql://testuser:testpassword@127.0.0.1:5432/database?sslmode=require#test'
+
+
+@pytest.mark.parametrize('field', ['host', 'password', 'username', 'port'])
+def test_multi_url_build_hosts_set_with_single_value(field) -> None:
+    """Hosts can't be provided with any single url values."""
+    hosts = [
+        {'host': '127.0.0.1', 'password': 'testpassword', 'username': 'testuser', 'port': 5432},
+        {'host': '127.0.0.1', 'password': 'testpassword', 'username': 'testuser', 'port': 5432},
+    ]
+    kwargs = dict(scheme='postgresql', hosts=hosts, path='database', query='sslmode=require', fragment='test')
+    if field == 'port':
+        kwargs[field] = 5432
+    else:
+        kwargs[field] = 'test'
+    with pytest.raises(ValueError):
+        MultiHostUrl.build(**kwargs)
+
+
+def test_multi_url_build_hosts_empty_host() -> None:
+    """Hosts can't be provided with any single url values."""
+    hosts = [{}]
+    with pytest.raises(ValueError):
+        MultiHostUrl.build(scheme='postgresql', hosts=hosts, path='database', query='sslmode=require', fragment='test')
+
+
+def test_multi_url_build_hosts() -> None:
+    """Hosts can't be provided with any single url values."""
+    hosts = [
+        {'host': '127.0.0.1', 'password': 'testpassword', 'username': 'testuser', 'port': 5431},
+        {'host': '127.0.0.1', 'password': 'testpassword', 'username': 'testuser', 'port': 5433},
+    ]
+    kwargs = dict(scheme='postgresql', hosts=hosts, path='database', query='sslmode=require', fragment='test')
+    url = MultiHostUrl.build(**kwargs)
+    assert url == MultiHostUrl(
+        'postgresql://testuser:testpassword@127.0.0.1:5431,testuser:testpassword@127.0.0.1:5433/database?sslmode=require#test'
+    )
+    assert (
+        str(url)
+        == 'postgresql://testuser:testpassword@127.0.0.1:5431,testuser:testpassword@127.0.0.1:5433/database?sslmode=require#test'
+    )
+
+
+def test_multi_url_build_neither_host_and_hosts_set() -> None:
+    with pytest.raises(ValueError):
+        MultiHostUrl.build(
+            scheme='postgresql',
+            username='testuser',
+            password='testpassword',
+            port=5432,
+            path='database',
+            query='sslmode=require',
+            fragment='test',
+        )
+
+
+def test_url_build() -> None:
+    url = Url.build(
+        scheme='postgresql',
+        username='testuser',
+        password='testpassword',
+        host='127.0.0.1',
+        port=5432,
+        path='database',
+        query='sslmode=require',
+        fragment='test',
+    )
+    assert url == Url('postgresql://testuser:testpassword@127.0.0.1:5432/database?sslmode=require#test')
+    assert str(url) == 'postgresql://testuser:testpassword@127.0.0.1:5432/database?sslmode=require#test'
