@@ -3,7 +3,7 @@ import platform
 import sys
 import weakref
 from collections import deque
-from typing import Any, Callable, Dict, List, Union, cast
+from typing import Any, Callable, Union, cast
 
 import pytest
 
@@ -164,7 +164,11 @@ def test_dict_keys():
 
 def test_tuple_variable(py_and_json: PyAndJson):
     v = py_and_json(
-        {'type': 'tuple-variable', 'items_schema': {'type': 'default', 'schema': {'type': 'int'}, 'on_error': 'omit'}}
+        {
+            'type': 'tuple',
+            'items_schema': [{'type': 'default', 'schema': {'type': 'int'}, 'on_error': 'omit'}],
+            'variadic_item_index': 0,
+        }
     )
     assert v.validate_python((1, 2, 3)) == (1, 2, 3)
     assert v.validate_python([1, '2', 3]) == (1, 2, 3)
@@ -174,7 +178,7 @@ def test_tuple_variable(py_and_json: PyAndJson):
 def test_tuple_positional():
     v = SchemaValidator(
         {
-            'type': 'tuple-positional',
+            'type': 'tuple',
             'items_schema': [{'type': 'int'}, {'type': 'default', 'schema': {'type': 'int'}, 'default': 42}],
         }
     )
@@ -187,9 +191,13 @@ def test_tuple_positional():
 def test_tuple_positional_omit():
     v = SchemaValidator(
         {
-            'type': 'tuple-positional',
-            'items_schema': [{'type': 'int'}, {'type': 'int'}],
-            'extras_schema': {'type': 'default', 'schema': {'type': 'int'}, 'on_error': 'omit'},
+            'type': 'tuple',
+            'items_schema': [
+                {'type': 'int'},
+                {'type': 'int'},
+                {'type': 'default', 'schema': {'type': 'int'}, 'on_error': 'omit'},
+            ],
+            'variadic_item_index': 2,
         }
     )
     assert v.validate_python((1, '2')) == (1, 2)
@@ -219,12 +227,18 @@ def test_factory_runtime_error():
         v.validate_python('wrong')
 
 
-def test_factory_type_error():
+def test_factory_missing_arg():
     def broken(x):
         return 7
 
     v = SchemaValidator(
-        {'type': 'default', 'schema': {'type': 'int'}, 'on_error': 'default', 'default_factory': broken}
+        {
+            'type': 'default',
+            'schema': {'type': 'int'},
+            'on_error': 'default',
+            'default_factory': broken,
+            'default_factory_takes_data': False,
+        }
     )
     assert v.validate_python(42) == 42
     assert v.validate_python('42') == 42
@@ -423,8 +437,8 @@ def test_deepcopy_mutable_defaults():
     stored_empty_dict = {}
 
     class Model:
-        int_list_with_default: List[int] = stored_empty_list
-        str_dict_with_default: Dict[str, str] = stored_empty_dict
+        int_list_with_default: list[int] = stored_empty_list
+        str_dict_with_default: dict[str, str] = stored_empty_dict
 
     v = SchemaValidator(
         {
